#!/bin/bash
#
# Copyright (c) 2010-2013, Novell, Inc.  All rights reserved.
# USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE DEVELOPER LICENSE
# AGREEMENT OR OTHER AGREEMENT THROUGH WHICH NOVELL, INC. MAKES THE WORK
# AVAILABLE. THIS WORK MAY NOT BE ADAPTED WITHOUT  NOVELL'S PRIOR WRITTEN
# CONSENT.
#
# NOVELL PROVIDES THE WORK "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
# INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY,FITNESS FOR
# A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  NOVELL, THE AUTHORS OF THE
# WORK, AND THE OWNERS OF COPYRIGHT IN THE WORK ARE NOT LIABLE FOR ANY
# CLAIM, DAMAGES, OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT, OR OTHERWISE, ARISING FROM, OUT OF, OR IN CONNECTION WITH THE WORK
# OR THE USE OR OTHER DEALINGS IN THE WORK.
#
# All rights reserved.
#
#

LDAP_MOD_BIN="env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapmodify"
LDAP_ADD_BIN="env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapmodify -a"
LDAP_SEARCH_BIN="env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapsearch"
LDAP_SEARCH_BIN_NOSSLPORT="/usr/bin/ldapsearch"
LDAP_SEARCH_BIN_NOCERT="env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem LDAPTLS_REQCERT=never /usr/bin/ldapsearch"
NMAS_ADD_BIN="/opt/novell/eDirectory/bin/nmasinst -addmethod"
FILE="/etc/opt/novell/cifs/cifsctxs.conf"
LOG_FILE="/var/opt/novell/log/proxymgmt/pxymgmt.log"
CIFS_YAST_FILE="/etc/sysconfig/novell/NvlCifs_oes2018_sp3"
LDAP_LISTENERS_FILE="/var/opt/novell/eDirectory/data/ldap_listeners.txt"

#Common Proxy framework will export admin credentials as OES_ADMIN_DATA and YAST will export the same as OES_DATA.
#So, if OES_ADMIN_DATA is not null then set OES_DATA to OES_ADMIN_DATA. Otherwise (in case if called from YAST) OES_DATA will

SchemaList=( nfapCIFSServerName nfapCIFSComment nfapCIFSWorkGroup nfapCIFSPDCName nfapCIFSAuthent nfapCIFSDialect nfapCIFSDebug nfapCIFSUnicode nfapCIFSOpLocks nfapCIFSAsync nfapCIFSWalk nfapCIFSPDCAddr nfapCIFSWINSAddr nfapCIFSAttach nfapCIFSNDSUserContext nfapCIFSUserContext nfapCIFSShares nfapCIFSDFS nfapCIFSLoginScripts nfapCIFSShareVolsByDefault nfapCIFSDomainDN nfapCIFSBeginRID nfapCIFSEndRID nfapCIFSPDCEnable nfapCIFSSignatures nfapLoginScript nfapCIFSRID nfapCIFSComment nfapCIFSNextRID nfapCIFSDomainSID nfapCIFSPDC nfapCIFSDCList nfapCIFSDCGroup nfapCIFSDomainEpoch )

#Check for root access
user_id=`id -u`
if test $user_id -ne 0
then
	echo "cifs_proxy_rights_assign.sh: Only root user can run this script" >> $LOG_FILE
	echo "cifs_proxy_rights_assign.sh: Only root user can run this script"
	exit 1
fi

#Help of the script
#######################################################################
#NOTE:If the script is executed with option to not grant proxy rights
#over NCP Server, then CIFS Server won't come up as it can't read 
#configuration. So, before starting CIFS Server, proxy user must be
#granted rights required over NCP Server.
#######################################################################
Usage()
{
	echo ""
	echo "Usage: cifs_proxy_rights_assign.sh [options]"
	echo "          -h                      Prints this summary"
	echo "          -d                      LDAP Admin FDN"
	echo "          -u                      Proxy DN in LDAP format"
	echo "          -i                      LDAP Server IP address"
	echo "          -p                      LDAP Port"
	echo "          -s                      SSL or Not(1 or 0). Default 1"
	echo "          -r                      (Optional) 0,1,2,3. Default 3"
	echo "					0 - Don't add proxy user as trustee of NCP Server and user contexts"
	echo "					1 - Add Proxy user as trustee of NCP Server only"
	echo "					2 - Add Proxy user as trustee of User Contexts only"
	echo "					3 - Add Proxy user as trustee of both NCP Server and User Contexts"
	echo ""
	exit 1;
}

#Read the command line arguments and store them
while getopts d:i:p:u:s:r:h o
do
        case "$o" in
			d)LDAP_ADM="$OPTARG";;
			i)LDAP_IP="$OPTARG";;
			p)LDAP_PORT="$OPTARG";;
			u)PROXYUSER="$OPTARG";;
			s)SSLORNOT="$OPTARG";;
			r)ADDTRUSTEE="$OPTARG";;
			h)Usage;;
			[?]) Usage;;
        esac
done

if test "$LDAP_ADM" = "" || test "$LDAP_IP" = "" || test "$LDAP_PORT" = "" || test "$PROXYUSER" = "" || test "$SSLORNOT" = "" 
then
        Usage
fi

if [ -n "$OES_ADMIN_DATA" ]; then
	OES_DATA="$OES_ADMIN_DATA"
fi
if [ -n "$OES_CIFS_DATA" ]; then
        OES_DATA2="$OES_CIFS_DATA"
fi

rightsornot=3	#Default value for rights assignment
cifsProxyUser=$PROXYUSER
cifsPassword="$OES_DATA2"
logindn=$LDAP_ADM
password="$OES_DATA"
hostname=$LDAP_IP
port=$LDAP_PORT
ssl=$SSLORNOT
localLDAPIP="127.0.0.1"
localLDAPSIP="127.0.0.1"
localLDAPPort="389"
localLDAPSPort="636"

#fetch local ldap IPs from ldap_listeners.txt
fetch_local_ldap_ip_port()
{
  if [ -f "$LDAP_LISTENERS_FILE" ]
  then
        ldapIPPort=`awk '{if ($1 == "n4u.ldap.interfaces.ldap") print $3}' "$LDAP_LISTENERS_FILE"`
        if [ ! -z ldapIPPort ]
        then
             IFS=':' read -r -a ipArray <<< "$ldapIPPort"
             if  [ ! -z ${ipArray[1]} ] && [ ${ipArray[0]} != "0.0.0.0" ]
             then
                  localLDAPIP=${ipArray[0]}
                  localLDAPPort=${ipArray[1]}
             fi
        fi
        ldapsIPPort=`awk '{if ($1 == "n4u.ldap.interfaces.ldaps") print $3}' "$LDAP_LISTENERS_FILE"`
        if [ ! -z ldapsIPPort ]
        then
             IFS=':' read -r -a ipArray <<< "$ldapsIPPort"
             if [ ! -z ${ipArray[1]} ] && [ ${ipArray[0]} != "0.0.0.0" ]
             then
                  localLDAPSIP=${ipArray[0]}
                  localLDAPSPort=${ipArray[1]}
             fi
        fi
  fi
}

# Grant rights for proxy user over user contexts, read from the cifsctxs.conf file
# Read and Compare rights over CN attribute for Subtree Search feature
assign_rights_context()
{
   TMP_LDIF_TRUSTEE="/tmp/cifs_trustee.ldif"

   mkdir -p /tmp
   touch $TMP_LDIF_TRUSTEE
   ATTR_RDCMP="03"
   ATTR_INHERIT="01"

   echo "dn:$context" > $TMP_LDIF_TRUSTEE
   echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
   echo "add: acl" >> $TMP_LDIF_TRUSTEE
   echo "acl: $ATTR_RDCMP#subtree#$cifsProxyUser#CN" >> $TMP_LDIF_TRUSTEE

   echo "Setting context trustee" >> $LOG_FILE

   if [ $ssl != 0 ]
   then
	   var=`env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapmodify -a -f $TMP_LDIF_TRUSTEE -c -H ldaps://"$hostname":$port -D "$logindn"  -w "$password"` >>$LOG_FILE 2>&1
#	   var=`$LDAP_ADD_BIN -f $TMP_LDIF_TRUSTEE -c -H ldaps://"$hostname":$port -D "$logindn"  -w "$password"` >>$LOG_FILE 2>&1
   else
	   var=`env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapmodify -a -f $TMP_LDIF_TRUSTEE -c -H ldap://"$hostname":$port -D "$logindn"  -w "$password"` >>$LOG_FILE 2>&1
   fi
   contextTrusteeRet=$?
   if test $contextTrusteeRet == 20
   then
	   echo "Proxy user $cifsProxyUser is already trustee of container $context" >> $LOG_FILE
       contextTrusteeRet=0;
   fi
   echo -e "Setting context trustee done $var \n Return Value : $contextTrusteeRet \n" >> $LOG_FILE
   rm -f $TMP_LDIF_TRUSTEE
}

#Grant rights for Proxy user on NCP Server Object - Entry Rights and All Attribute Rights
set_trustee() {

        TMP_LDIF_TRUSTEE="/tmp/cifs_trustee.ldif"
        mkdir -p /tmp
        touch $TMP_LDIF_TRUSTEE
        ATTR_SUPERVISOR="32"
        ATTR_RDCMP="07"
        ATTR_RD_CMP="03"

        fetch_local_ldap_ip_port

        if [ $ssl != 0 ]
        then
                NCP_Server_Object=`$LDAP_SEARCH_BIN_NOCERT  -H ldaps://$localLDAPSIP:$localLDAPSPort -D "$logindn" -w "$password" -b "" -x -s base dsaName -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //g; }; ' | cut -c 10- | grep -vE '^$'`
        else
                NCP_Server_Object=`$LDAP_SEARCH_BIN_NOSSLPORT -H ldap://$localLDAPIP:$localLDAPPort -D "$logindn" -w "$password" -b "" -s base dsaName -LLL | sed ' /^ / {; H; d; }; /^ /! {; x; s/\n //g; }; ' | cut -c 10- | grep -vE '^$'`
        fi

        echo -e "Setting Trustee \n NCP Server Object : $NCP_Server_Object" >> $LOG_FILE
	
	if [ $ssl != 0 ]
	then
        	currentAAR=`$LDAP_SEARCH_BIN -H ldaps://"$hostname":$port -D "$logindn" -w "$password" -b "$NCP_Server_Object" -s base -x -LLL  ACL | grep -i "$cifsProxyUser" | grep -i "All Attributes Rights" | grep -i subtree | tr -d ' ' | awk -F# '{print $1}' | cut -c 5-`
	else
		currentAAR=`$LDAP_SEARCH_BIN -H ldap://"$hostname":$port -D "$logindn" -w "$password" -b "$NCP_Server_Object" -s base -x -LLL  ACL | grep -i "$cifsProxyUser" | grep -i "All Attributes Rights" | grep -i subtree | tr -d ' ' | awk -F# '{print $1}' | cut -c 5-`
	fi

        cifsAAR=`expr $currentAAR + $ATTR_RDCMP`
        echo "[DEBUG]cifsAAR is   : $cifsAAR" >> $LOG_FILE
	echo "[DEBUG]currentAAR is: $currentAAR" >> $LOG_FILE
        echo "$NCP_Server_Object"
        echo "-------------------\n"
	
	echo "dn: $NCP_Server_Object"
	echo "dn: $NCP_Server_Object" > $TMP_LDIF_TRUSTEE
	echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
	echo "delete: acl" >> $TMP_LDIF_TRUSTEE

	if [ -z $currentAAR ]
	then
		echo "acl: 0#subtree#$cifsProxyUser#[All Attributes Rights]" >> $TMP_LDIF_TRUSTEE
	else
		echo "acl: $currentAAR#subtree#$cifsProxyUser#[All Attributes Rights]" >> $TMP_LDIF_TRUSTEE
	fi

	if [ $ssl != 0 ]
	then
		var=`$LDAP_MOD_BIN -f $TMP_LDIF_TRUSTEE -H ldaps://"$hostname":$port -D "$logindn"  -w "$password"` >>$LOG_FILE 2>&1
	else
		var=`$LDAP_MOD_BIN -f $TMP_LDIF_TRUSTEE -H ldap://"$hostname":$port -D "$logindn"  -w "$password"` >>$LOG_FILE 2>&1
	fi

        echo "dn: $NCP_Server_Object" > $TMP_LDIF_TRUSTEE
        echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
        echo "add: acl" >> $TMP_LDIF_TRUSTEE
        echo "acl: $cifsAAR#subtree#$cifsProxyUser#[All Attributes Rights]" >> $TMP_LDIF_TRUSTEE

                for name in ${SchemaList[@]}
        do
                echo "" >> $TMP_LDIF_TRUSTEE
                echo "dn: $NCP_Server_Object" >> $TMP_LDIF_TRUSTEE
                echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
                echo "add: acl" >> $TMP_LDIF_TRUSTEE
                echo "acl: $ATTR_SUPERVISOR#entry#$cifsProxyUser#$name" >> $TMP_LDIF_TRUSTEE
        done
                echo "" >> $TMP_LDIF_TRUSTEE
                echo "dn: $NCP_Server_Object" >> $TMP_LDIF_TRUSTEE
                echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
                echo "add: acl" >> $TMP_LDIF_TRUSTEE
                echo "acl: $ATTR_RD_CMP#entry#[Root]#nfapCIFSServerName" >> $TMP_LDIF_TRUSTEE

        if [ $ssl != 0 ]
        then
                var=`$LDAP_MOD_BIN -x -c -a -H ldaps://"$hostname":$port -D "$logindn"  -w "$password" -f $TMP_LDIF_TRUSTEE ` >>$LOG_FILE 2>&1
        else
                var=`$LDAP_MOD_BIN -x -c -a -H ldap://"$hostname":$port -D "$logindn"  -w "$password" -f  $TMP_LDIF_TRUSTEE` >>$LOG_FILE 2>&1
        fi
        trusteeRet=$?
        echo -e "[cifs_proxy_rights_assign.sh] set_trustee function Completed . Return Value : $trusteeRet" >> $LOG_FILE
        rm -f $TMP_LDIF_TRUSTEE
}

#Grant rights for Proxy user over nfapCIFS Attributes of NCP Server
setnfapTrustee() {

        TMP_LDIF_TRUSTEE="/tmp/cifs_trustee.ldif"
        mkdir -p /tmp
        touch $TMP_LDIF_TRUSTEE

        ATTR_RDCMP="07"
        ATTR_INHERIT="01"

	if [ $ssl != 0 ]
	then
        	currentAcl=`$LDAP_SEARCH_BIN -H ldaps://"$hostname":$port  -D "$logindn" -w "$password" -b "$serverContext" -s base -x -LLL  ACL | grep -i "$cifsProxyUser" | grep -i "Entry Rights" | grep -i subtree | tr -d ' ' | awk -F# '{print $1}' | cut -c 5-`
	else
		currentAcl=`$LDAP_SEARCH_BIN -H ldap://"$hostname":$port  -D "$logindn" -w "$password" -b "$serverContext" -s base -x -LLL  ACL | grep -i "$cifsProxyUser" | grep -i "Entry Rights" | grep -i subtree | tr -d ' ' | awk -F# '{print $1}' | cut -c 5-`
	fi

        cifsAcl=`expr $currentAcl + 1`
	echo "[DEBUG]setnfapTrustee: currentAcl value : $currentAcl" >> $LOG_FILE
	echo "[DEBUG]setnfapTrustee: cifsAcl value: $cifsAcl" >>$LOG_FILE

	echo "dn:$serverContext" > $TMP_LDIF_TRUSTEE
	echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
	echo "delete: acl" >> $TMP_LDIF_TRUSTEE

	if [ -z $currentAcl ]
	then
		echo "acl: 0#subtree#$cifsProxyUser#[Entry Rights]" >> $TMP_LDIF_TRUSTEE
	else
		echo "acl: $currentAcl#subtree#$cifsProxyUser#[Entry Rights]" >> $TMP_LDIF_TRUSTEE
	fi

	if [ $ssl != 0 ]
	then
		var=`$LDAP_MOD_BIN -f $TMP_LDIF_TRUSTEE -H ldaps://"$hostname":$port -D "$logindn" -w "$password" ` >>$LOG_FILE 2>&1
	else
		var=`$LDAP_MOD_BIN -f $TMP_LDIF_TRUSTEE -H ldap://"$hostname":$port -D "$logindn" -w "$password"` >>$LOG_FILE 2>&1
	fi

        echo "dn:$serverContext" > $TMP_LDIF_TRUSTEE
        echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
        echo "add: acl" >> $TMP_LDIF_TRUSTEE
        echo "acl: $ATTR_RDCMP#subtree#$cifsProxyUser#nfapCIFSComment" >> $TMP_LDIF_TRUSTEE

        echo "" >> $TMP_LDIF_TRUSTEE
        echo "dn:$serverContext" >> $TMP_LDIF_TRUSTEE
        echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
        echo "add: acl" >> $TMP_LDIF_TRUSTEE
        echo "acl: $ATTR_RDCMP#subtree#$cifsProxyUser#nfapCIFSServerName" >> $TMP_LDIF_TRUSTEE

        echo "" >> $TMP_LDIF_TRUSTEE
        echo "dn:$serverContext" >> $TMP_LDIF_TRUSTEE
        echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
        echo "add: acl" >> $TMP_LDIF_TRUSTEE
        echo "acl: $ATTR_RDCMP#subtree#$cifsProxyUser#nfapCIFSAttach" >> $TMP_LDIF_TRUSTEE

        echo "" >> $TMP_LDIF_TRUSTEE
        echo "dn:$serverContext" >> $TMP_LDIF_TRUSTEE
        echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
        echo "add: acl" >> $TMP_LDIF_TRUSTEE
        echo "acl: $ATTR_RDCMP#subtree#$cifsProxyUser#nfapCIFSShares" >> $TMP_LDIF_TRUSTEE

        echo "" >> $TMP_LDIF_TRUSTEE
        echo "dn:$serverContext" >> $TMP_LDIF_TRUSTEE
        echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
        echo "add: acl" >> $TMP_LDIF_TRUSTEE
        echo "acl: $ATTR_RD_CMP#subtree#$cifsProxyUser#CN" >> $TMP_LDIF_TRUSTEE
        echo "" >> $TMP_LDIF_TRUSTEE

        echo "" >> $TMP_LDIF_TRUSTEE
        echo "dn:$serverContext" >> $TMP_LDIF_TRUSTEE
        echo "changetype: modify" >> $TMP_LDIF_TRUSTEE
        echo "add: acl" >> $TMP_LDIF_TRUSTEE
#        echo "acl: $ATTR_INHERIT#subtree#$cifsProxyUser#[Entry Rights]" >> $TMP_LDIF_TRUSTEE
        echo "acl: $cifsAcl#subtree#$cifsProxyUser#[Entry Rights]" >> $TMP_LDIF_TRUSTEE

        echo "Setting context trustee" >> $LOG_FILE

        if [ $ssl != 0 ]
        then
                var=`$LDAP_ADD_BIN -f $TMP_LDIF_TRUSTEE -c -H ldaps://"$hostname":$port -D "$logindn" -w "$password" ` >>$LOG_FILE 2>&1
        else
                var=`$LDAP_ADD_BIN -f $TMP_LDIF_TRUSTEE -c -H ldap://"$hostname":$port -D "$logindn" -w "$password"` >>$LOG_FILE 2>&1
        fi
        contextTrusteeRet=$?
        echo -e "setnfapTrustee function completed. Return Value : $contextTrusteeRet" >> $LOG_FILE
        rm -f $TMP_LDIF_TRUSTEE
}

# make sure user context file exists and readable
if [ ! -f $FILE ]; then
	echo "$FILE : does not exists" >> $LOG_FILE
	exit 2
elif [ ! -r $FILE ]; then
	echo "$FILE: can not read" >> $LOG_FILE
	exit 3
fi

serverContext=`/opt/novell/eDirectory/bin/ndsconfig get | grep n4u.nds.server-context | tr -d ' ' | cut -d"=" -f2- | tr . ,`
echo "Server Context is: $serverContext" >> $LOG_FILE

#If the option to grant rights to proxy user is passed
if test "$ADDTRUSTEE" != ""
then
	rightsornot=$ADDTRUSTEE
fi

#Grant rights over NCP Server Object
if test $rightsornot == 1 || test $rightsornot == 3
then
	echo "Adding proxy user $cifsProxyUser as trustee of NCP Server" >> $LOG_FILE
	set_trustee
	setnfapTrustee
else
	echo "Proxy user $cifsProxyUser is not added as trustee of  NCP Server" >> $LOG_FILE
fi

#Determine whether to grant rights to proxy user over user contexts
USERCONTEXT_RIGHTS=1
if test "$ADDTRUSTEE" != ""
then
	#Case 1 - Explicitly asked to do through command line parameter
	if test $rightsornot != 2 && test $rightsornot != 3
	then
		USERCONTEXT_RIGHTS=0
	fi
else
	#Case 2 - Selected at install time
	grep "CONFIG_CIFS_USERCONTEXT_RIGHTS" $CIFS_YAST_FILE | grep -i "no" >> /dev/null
	FOUNDOPTION=$?
	if test $FOUNDOPTION == 0
	then
		USERCONTEXT_RIGHTS=0
	fi
fi

# read user context file $FILE using the file descriptors
# Set loop separator to end of line

#Add proxy as trustee of user contexts or no
if test $USERCONTEXT_RIGHTS == 1
then
	echo "Adding Proxy user $cifsProxyUser as trustee of user contexts in the file $FILE" >> $LOG_FILE
	BAKIFS=$IFS
	IFS=$(echo -en "\n\b")
	exec 3<&0
	exec 0<$FILE
	while read line
	do
		c=`echo "$line" | sed -e 's/^[ \t]*//' | head -c 1`
		if [ "$c" = "#" ];then
			continue
		fi
		context=`echo $line | tr . , `
		assign_rights_context
		if test $contextTrusteeRet != 0; then
			echo "Proxy user $cifsProxyUser couldn't be given rights over context $context" >> $LOG_FILE
			exec 0<&3
			IFS=$BAKIFS
			exit $contextTrusteeRet
		fi
	done
	exec 0<&3
	# restore $IFS which was used to determine what the field separators are
	IFS=$BAKIFS
else
	echo "Proxy user $cifsProxyUser is not added as trustee of user contexts" >> $LOG_FILE
fi

unset OES_DATA
unset OES_DATA2
exit 0
