-- Novell Native File Access Package (NFAP) ASN1 Schema Definitions
--
-- Novell Inc.
-- 1800 South Novell Place
-- Provo, UT 84606-6194

-- Version=3.00
-- Copyright=(c) Copyright 2008, Novell, Inc.  All rights reserved

-- Object ID Syntax
------------------------------------------
------------------------------------------
-- Change History:
--
-- Thu, Oct 18, 2001:  Initial revision
--  *** We do not yet have an official OID assigned ***
--  *** The nfap prefix is not yet officially registered ***
-- Mon, Nov 19, 2001:  
--  *** We still do not yet have an official OID assigned ***
--  *** Removed DS_PUBLIC_READP from all attributes except server name***
--  *** Recommendation from SRB to remove DS_SYNC_IMMEDIATE, but 
--  *** consistency with Cluster and infrequent updates suggests that
--  *** the DS_SYNC_IMMEDIATE stays for all attributes ***
--  *** Added and cleaned up comments for attributes and Aux Class ***
-- Mon, Dec 03, 2001
--  *** Replaced 138 with the real number 277   
-- Fri, Jan 24, 2003
--  *** Added new Boolean attributes to nfapCIFSConfigInfo class:
--  *** nfapCIFSDFS and nfapCIFSLoginScripts. Also added aux class
--  *** nfapLoginProperties which may contain new Stream attribute
--  *** nfapLoginScript.
-- Mon, Dec 15, 2003
--  *** Added new Boolean nfapCIFSShareVolsByDefault
--  *** attribute to nfapCIFSConfigInfo aux class. Also added 
--  *** nfapCIFSDomainDN attribute, nfapCIFSBeginRID and nfapCIFSEndRID integer attributes.
--  *** Added new typed name nfapCIFSRID attribute to nfapLoginProperties class.
--  *** Added aux class nfapCIFSDomainProperties aux class which may contain 
--  *** nfapCIFSNextRID attribute, nfapCIFSDomainSID Octet-string attribute, 
--  *** nfapCIFSPDC and NfapCIFSDCList DN attributes, nfapCIFSDCGroup DN attribute,
--  *** and integer attribute nfapCIFSDomainEpoch
-- Mon, May 10, 2004
--  *** Added new Boolean nfapCIFSPDCEnable
-- Wed, May 4, 2005
--  *** Added new Integer nfapCIFSSignatures attribute used to determine status
--  *** of SMB signing.
--
-- Naming Convention: nfap prefix to all NFAP attributes and classes
--                    followed by service name (all caps)
--
------------------------------------------
------------------------------------------
--
-- ASN1 Object ID's Defined as Follows:
--     joint-iso-ccitt(2) country(16) us(840) organization(1)
--     Novell(113719) applications(1) NFAP(???)
--
--   components:                      NFAPAFP(1)
--                                    NFAPCIFS(2)
--                                    NFAPNFS(3)
--
--                                    NFAPAttributeType(4)
--                                    NFAPClass(6)
--
------------------------------------------
-- 
-- Attribute range 1-100 is reserved for this file.
-- Class range 1-10 is reserved for this file.
-- 
------------------------------------------


NFAPSchemaExtensions DEFINITIONS ::=
BEGIN

-- New Attributes (generic NFAP)ed
------------------------------------------
------------------------------------------

-- The server name that CIFS clients use to access this machine
"nfapCIFSServerName" ATTRIBUTE ::=
{
  SyntaxID    SYN_CI_STRING,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_PUBLIC_READ, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 1}
}

-- Comment associated with this CIFS server (optional)
"nfapCIFSComment" ATTRIBUTE ::=
{
  SyntaxID    SYN_CI_STRING,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 2}
}

-- Name of the workgroup or domain to which the CIFS server belongs.
"nfapCIFSWorkGroup" ATTRIBUTE ::=
{
  SyntaxID    SYN_CI_STRING,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 3}
}

-- WINS name of the Primary Domain Controller (PDC), used when CIFS and PDC are on the same subnet
"nfapCIFSPDCName" ATTRIBUTE ::=
{
  SyntaxID    SYN_CI_STRING,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 4}
}

-- Authentication mode, 1 for "local" or 2 for "domain"
"nfapCIFSAuthent" ATTRIBUTE ::=
{
  SyntaxID    SYN_INTEGER,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 5}
}

-- Specifies the dialect of the protocol to use, 0 for "PC"  4 for "LM_NT_12"
"nfapCIFSDialect" ATTRIBUTE ::=
{
  SyntaxID    SYN_INTEGER,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 6}
}

-- Hex value used to turn on more debug trace info
"nfapCIFSDebug" ATTRIBUTE ::=
{
  SyntaxID    SYN_INTEGER,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 7}
}

-- Whether UNICODE strings are enabled or not
"nfapCIFSUnicode" ATTRIBUTE ::=
{
  SyntaxID    SYN_BOOLEAN,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 8}
}

-- Whether Oplocks are enabled or not
"nfapCIFSOpLocks" ATTRIBUTE ::=
{
  SyntaxID    SYN_BOOLEAN,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 9}
}

-- Whether Asynchronous Reads are enabled or not
"nfapCIFSAsync" ATTRIBUTE ::=
{
  SyntaxID    SYN_BOOLEAN,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 10}
}

-- Whether full the full pathname of files is walked or not
"nfapCIFSWalk" ATTRIBUTE ::=
{
  SyntaxID    SYN_BOOLEAN,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 11}
}

-- IP address of the PDC.  Needed when CIFS and PDC are not on the same subnet
"nfapCIFSPDCAddr" ATTRIBUTE ::=
{
  SyntaxID    SYN_NET_ADDRESS,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 12}
}

-- IP Address of the WINS server to talk to
"nfapCIFSWINSAddr" ATTRIBUTE ::=
{
  SyntaxID    SYN_NET_ADDRESS,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 13}
}

-- Set of IP addresses to enable for CIFS
"nfapCIFSAttach" ATTRIBUTE ::=
{
  SyntaxID    SYN_NET_ADDRESS,
  Flags       {DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 14}
}

-- The context(s) in the tree into which NT Domain users are imported, used in DOMAIN mode
"nfapCIFSNDSUserContext" ATTRIBUTE ::=
{
  SyntaxID    SYN_CI_LIST,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 15}
}

-- The set of contexts in the tree to look for User objects, used in LOCAL and DOMAIN mode
"nfapCIFSUserContext" ATTRIBUTE ::=
{
  SyntaxID    SYN_CI_LIST,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 16}
}

-- The list of "share points" (volumes) exported by this server
"nfapCIFSShares" ATTRIBUTE ::=
{
  SyntaxID    SYN_CI_LIST,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 17}
}

-- Whether DFS is enabled or not
"nfapCIFSDFS" ATTRIBUTE ::=
{
  SyntaxID    SYN_BOOLEAN,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 18}
}

-- Whether login scripts are enabled or not
"nfapCIFSLoginScripts" ATTRIBUTE ::=
{
  SyntaxID    SYN_BOOLEAN,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 19}
}

-- NFAP login script stream
"nfapLoginScript" ATTRIBUTE ::=
{
  SyntaxID    SYN_STREAM,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 20}
}


-- 32-byte Relative ID field (RID) for domain users and groups
"nfapCIFSRID" ATTRIBUTE ::=
{
  SyntaxID    SYN_TYPED_NAME,
  Flags       {DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 21}
}

-- 32-byte Next Available Relative ID (RID) field used for unique RID assignment.
"nfapCIFSNextRID" ATTRIBUTE ::=
{
  SyntaxID    SYN_INTEGER,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 22}
}

-- Domain Security ID (SID)
"nfapCIFSDomainSID" ATTRIBUTE ::=
{
  SyntaxID    SYN_OCTET_STRING,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 23}
}

-- Primary Domain Controller DN
"nfapCIFSPDC" ATTRIBUTE ::=
{
  SyntaxID    SYN_DIST_NAME,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 24}
}

-- Domain Controller DN List
"nfapCIFSDCList" ATTRIBUTE ::=
{
  SyntaxID    SYN_DIST_NAME,
  Flags       {DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 25}
}

-- Domain Controller Group Object DN
"nfapCIFSDCGroup" ATTRIBUTE ::=
{
  SyntaxID    SYN_DIST_NAME,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 26}
}

-- CIFS Domain Epoch Number.
"nfapCIFSDomainEpoch" ATTRIBUTE ::=
{
  SyntaxID    SYN_INTEGER,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 27}
}

-- Whether we always share all mounted volumes by default or not
"nfapCIFSShareVolsByDefault" ATTRIBUTE ::=
{
  SyntaxID    SYN_BOOLEAN,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 28}
}

-- Domain Object DN
"nfapCIFSDomainDN" ATTRIBUTE ::=
{
  SyntaxID    SYN_DIST_NAME,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 29}
}

-- 32-byte value determining what begin value server can use to assign RIDs.
"nfapCIFSBeginRID" ATTRIBUTE ::=
{
  SyntaxID    SYN_INTEGER,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 30}
}

-- 32-byte value determining what end value server can use to assign RIDs.
"nfapCIFSEndRID" ATTRIBUTE ::=
{
  SyntaxID    SYN_INTEGER,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 31}
}

-- Whether CIFS PDC Add-on is enabled or not
"nfapCIFSPDCEnable" ATTRIBUTE ::=
{
  SyntaxID    SYN_BOOLEAN,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 32}
}

-- Server SMB Signing status, 0 = Disabled, 1 = Optional, 2 = Mandatory
"nfapCIFSSignatures" ATTRIBUTE ::=
{
  SyntaxID    SYN_INTEGER,
  Flags       {DS_SINGLE_VALUED_ATTR, DS_SYNC_IMMEDIATE},
  ASN1ObjID   {2 16 840 1 113719 1 277 2 4 33}
}


-- Effective Class Modification 
------------------------------------------
------------------------------------------
-- (none)

-- New/Modified Aux Classes (generic NFAP)
------------------------------------------
------------------------------------------

-- The auxiliary class that holds the CIFS configuration attributes
-- This Aux Class will will be applied to the NCP Server class
"nfapCIFSConfigInfo" OBJECT-CLASS ::=
{
    Operation   ADD,
    Flags       {DS_AUXILIARY_CLASS},
    MayContain  {
        "nfapCIFSServerName",
        "nfapCIFSComment",
        "nfapCIFSWorkGroup",
        "nfapCIFSPDCName",

        "nfapCIFSAuthent",
        "nfapCIFSDialect", 
        "nfapCIFSDebug",

        "nfapCIFSUnicode",
        "nfapCIFSOpLocks",
        "nfapCIFSAsync",
        "nfapCIFSWalk",

        "nfapCIFSPDCAddr",
        "nfapCIFSWINSAddr",
        "nfapCIFSAttach",

        "nfapCIFSNDSUserContext",
        "nfapCIFSUserContext",
        "nfapCIFSShares",
	"nfapCIFSDFS",
	"nfapCIFSLoginScripts",

	"nfapCIFSShareVolsByDefault",
	"nfapCIFSDomainDN",
	"nfapCIFSBeginRID",
	"nfapCIFSEndRID",
	"nfapCIFSPDCEnable",
	"nfapCIFSSignatures"
    },
    ASN1ObjID   {2 16 840 1 113719 1 277 2 6 1}
}

-- The auxiliary class that holds the NFAP login attributes
-- This Aux Class will will be applied to the User, Container,
-- Group, and Profile classes
"nfapLoginProperties" OBJECT-CLASS ::=
{
    Operation   ADD,
    Flags       {DS_AUXILIARY_CLASS},
    MayContain  {
	"nfapLoginScript",
	"nfapCIFSRID"
    },
    ASN1ObjID   {2 16 840 1 113719 1 277 2 6 2}
}

-- The auxiliary class that holds the NFAP CIFS domain attributes
-- This Aux Class will be applied to Domain Container Objects
"nfapCIFSDomainProperties" OBJECT-CLASS ::=
{
    Operation   ADD,
    Flags       {DS_AUXILIARY_CLASS},
    MayContain  {
	"nfapCIFSComment",
	"nfapCIFSNextRID",
	"nfapCIFSDomainSID",
	"nfapCIFSPDC",
	"nfapCIFSDCList",
	"nfapCIFSDCGroup",
	"nfapCIFSDomainEpoch"
    },
    ASN1ObjID   {2 16 840 1 113719 1 277 2 6 3}
}

END
